from fixieai.agents.api import AgentQuery
from fixieai.agents.api import AgentResponse
from fixieai.agents.api import Embed
from fixieai.agents.api import Message
from fixieai.agents.code_shot import CodeShotAgent

__all__ = [
    "Message",
    "Embed",
    "AgentQuery",
    "AgentResponse",
    "CodeShotAgent",
]



from fixieai.agents.api import AgentQuery
from fixieai.agents.api import AgentResponse
from fixieai.agents.api import Embed
from fixieai.agents.api import Message
from fixieai.agents.code_shot import CodeShotAgent
from fixieai.client import FixieClient
from fixieai.client import get_agents
from fixieai.client import get_client
from fixieai.client import get_embeds
from fixieai.client import query

__all__ = [
    "AgentQuery",
    "AgentResponse",
    "Embed",
    "Message",
    "CodeShotAgent",
    "FixieClient",
    "get_agents",
    "get_client",
    "get_embeds",
    "query",
]



import fastapi
import pytest
from fastapi import testclient

import fixieai
from fixieai import agents
from fixieai.agents import code_shot

BASE_PROMPT = "I am a simple dummy agent."
FEW_SHOTS = """
Q: Sample query 1
Ask Func[simple1]: Simple argument
Func[simple1] says: Simple response
A: Simple final response

Q: Sample query 2
Ask Func[simple2]: Simple argument
Func[simple2] says: Simple response
A: Simple final response
"""


@pytest.fixture
def dummy_agent():
    agent = agents.CodeShotAgent(BASE_PROMPT, FEW_SHOTS)

    @agent.register_func
    def simple1(query: agents.AgentQuery) -> str:
        return "Simple response 1"

    @agent.register_func()
    def simple2(query: agents.AgentQuery):
        return "Simple response 2"

    @agent.register_func(func_name="custom")
    def simple3(query):
        return "Simple response custom"

    return agent


def test_simple_agent_handshake(dummy_agent):
    fast_api = fastapi.FastAPI()
    fast_api.include_router(dummy_agent.api_router())
    client = testclient.TestClient(fast_api)
    response = client.get("/")
    assert response.status_code == 200
    json = response.json()
    assert json == {
        "base_prompt": dummy_agent.base_prompt,
        "few_shots": dummy_agent.few_shots,
    }


def test_simple_agent_func_calls(dummy_agent):
    fast_api = fastapi.FastAPI()
    fast_api.include_router(dummy_agent.api_router())
    client = testclient.TestClient(fast_api, raise_server_exceptions=False)

    # Test Func[simple1]
    response = client.post("/simple1", json={"message": {"text": "Howdy"}})
    assert response.status_code == 200
    json = response.json()
    assert json == {"message": {"text": "Simple response 1", "embeds": {}}}

    # Test Func[simple2]
    response = client.post("/simple2", json={"message": {"text": "Howdy"}})
    assert response.status_code == 200
    json = response.json()
    assert json == {"message": {"text": "Simple response 2", "embeds": {}}}

    # Test Func[custom]
    response = client.post("/custom", json={"message": {"text": "Howdy"}})
    assert response.status_code == 200
    json = response.json()
    assert json == {"message": {"text": "Simple response custom", "embeds": {}}}

    # Test non-existing Func[] returns 404: Not Found
    response = client.post("/simple3", json={"message": {"text": "Howdy"}})
    assert response.status_code == 404

    # Test Func[simple1] with bad arguments returns 422: Unprocessable Entity
    response = client.post("/simple1", json={"message": {"ttt": "Howdy"}})
    assert response.status_code == 422

    # Test Func[__init__] 404: Not Found
    response = client.post("/__init__", json={"message": {"text": "Howdy"}})
    assert response.status_code == 404


def test_split_few_shots():
    few_shots_list = code_shot._split_few_shots(FEW_SHOTS)
    assert few_shots_list == [
        """
Q: Sample query 1
Ask Func[simple1]: Simple argument
Func[simple1] says: Simple response
A: Simple final response""",
        """Q: Sample query 2
Ask Func[simple2]: Simple argument
Func[simple2] says: Simple response
A: Simple final response""",
    ]


def good_duck_typed_func(x):
    return "test"


def bad_duck_typed_func1(x, y):
    return "test"


def bad_duck_typed_func2(*x):
    return "test"


def bad_duck_typed_func3(**x):
    return "test"


def good_typed_func1(x: fixieai.AgentQuery) -> fixieai.AgentResponse:
    return fixieai.AgentResponse(fixieai.Message("test"))


def good_typed_func2(x: fixieai.AgentQuery) -> fixieai.Message:
    return fixieai.Message("test")


def good_typed_func3(x: fixieai.AgentQuery) -> str:
    return "test"


def good_semi_typed_func4(x: fixieai.AgentQuery):
    ...


def good_semi_typed_func5(x) -> str:
    return "test"


def bad_typed_func1(x: str) -> str:
    return "test"


def bad_typed_func2(x: int) -> str:
    return "test"


def test_registering_func(dummy_agent):
    dummy_agent.register_func(good_typed_func1, func_name="name1")
    dummy_agent.register_func(good_typed_func2, func_name="name2")
    dummy_agent.register_func(good_typed_func3, func_name="name3")
    assert dummy_agent._funcs["name1"] == good_typed_func1
    assert dummy_agent._funcs["name2"] == good_typed_func2
    assert dummy_agent._funcs["name3"] == good_typed_func3

    dummy_agent.register_func(good_duck_typed_func)
    assert dummy_agent._funcs["good_duck_typed_func"] == good_duck_typed_func

    dummy_agent.register_func(good_semi_typed_func4)
    assert dummy_agent._funcs["good_semi_typed_func4"] == good_semi_typed_func4

    dummy_agent.register_func(good_semi_typed_func5)
    assert dummy_agent._funcs["good_semi_typed_func5"] == good_semi_typed_func5

    with pytest.raises(TypeError):
        dummy_agent.register_func(bad_duck_typed_func1)
    with pytest.raises(TypeError):
        dummy_agent.register_func(bad_duck_typed_func2)
    with pytest.raises(TypeError):
        dummy_agent.register_func(bad_duck_typed_func3)
    with pytest.raises(TypeError):
        dummy_agent.register_func(bad_typed_func1)
    with pytest.raises(TypeError):
        dummy_agent.register_func(bad_typed_func2)



from __future__ import annotations

import functools
import re
import threading
from typing import Callable, Dict, List, Optional, Union

import fastapi
import requests
import uvicorn
from pydantic import dataclasses as pydantic_dataclasses

from fixieai import constants
from fixieai.agents import utils
from fixieai.agents.api import AgentQuery
from fixieai.agents.api import AgentResponse
from fixieai.agents.api import Message

# Regex that controls what Func names are allowed.
ACCEPTED_FUNC_NAMES = re.compile(r"^\w+$")


@pydantic_dataclasses.dataclass
class AgentMetadata:
    """Metadata for a Fixie CodeShot Agent.

    This will get sent to the Fixie platform upon handshake.
    """

    base_prompt: str
    few_shots: List[str]

    def __post_init__(self):
        utils.strip_prompt_lines(self)
        utils.validate_code_shot_agent(self)


class CodeShotAgent:
    """A CodeShot agent.

    To make a CodeShot agent, simply pass a BASE_PROMPT and FEW_SHOTS:

        BASE_PROMPT = "A summary of what this agent does; how it does it; and its
        personality"

        FEW_SHOTS = '''
        Q: <Sample query that this agent supports>
        A: <Desired response for this query>

        Q: <Another sample query>
        A: <Desired response for this query>
        '''

        agent = CodeShotAgent(BASE_PROMPT, FEW_SHOTS)

    You can have FEW_SHOTS as a single string of all your few-shots separated by 2 new
    lines, or as an explicit list of one few-shot per index.

    Your few-shots may reach out to other Agents in the fixie ecosystem by
    "Ask Agent[agent_id]: <query to pass>", or reach out to some python functions
    by "Ask Func[func_name]: <query to pass>".

    There are a series of default runtime `Func`s provided by the platform available for
    your agents to consume. For a full list of default runtime `Func`s, refer to:
        http://docs.fixie.ai/XXX

    You may also need to write your own python functions here to be consumed by your
    agent. To make a function accessible by an agent, you'd need to register it by
    `@agent.register_func`. Example:

        @agent.register_func
        def func_name(query: fixieai.AgentQuery) -> ReturnType:
            ...

        , where ReturnType is one of `str`, `fixieai.Message`, or `fixie.AgentResponse`.

    Note that in the above, we are using the decorator `@agent.register_func` to
    register this function with the agent instance we just created.

    To check out the default `Func`s that are provided in Fixie, see:
        http://docs.fixie.ai/XXX

    """

    def __init__(self, base_prompt: str, few_shots: Union[str, List[str]]):
        if isinstance(few_shots, str):
            few_shots = _split_few_shots(few_shots)

        self.base_prompt = base_prompt
        self.few_shots = few_shots
        self._funcs: Dict[str, Callable] = {}

    def serve(self, agent_id: str, host: str = "0.0.0.0", port: int = 8181):
        """Starts serving the current agent at `{host}:{port}` via uvicorn.

        This pings Fixie upon startup to fetch the latest prompt and fewshots.

        Args:
            agent_id: Fixie agent_id that this agent will serve.
            host: The address to start listening at.
            port: The port number to start listening at.
        """
        fast_api = fastapi.FastAPI()
        fast_api.include_router(self.api_router())
        fast_api.add_event_handler(
            "startup", functools.partial(_ping_fixie_async, agent_id)
        )
        uvicorn.run(fast_api, host=host, port=port)

    def api_router(self) -> fastapi.APIRouter:
        """Returns a fastapi.APIRouter object that serves the agent."""
        router = fastapi.APIRouter()
        router.add_api_route("/", self._handshake, methods=["GET"])
        router.add_api_route("/{func_name}", self._serve_func, methods=["POST"])
        return router

    def register_func(
        self, func: Optional[Callable] = None, *, func_name: Optional[str] = None
    ) -> Callable:
        """A function decorator to register `Func`s with this agent.

        This decorator will not change the callable itself.

        Usage:

            agent = CodeShotAgent(base_prompt, few_shots)

            @agent.register_func
            def func(query):
                ...

        Optional Decorator Args:
            func_name: Optional function name to register this function by. If unset,
                the function name will be used.
        """
        if func is None:
            # Func is not passed in. It's the decorator being created.
            return functools.partial(self.register_func, func_name=func_name)

        if func_name is not None:
            if not ACCEPTED_FUNC_NAMES.fullmatch(func_name):
                raise ValueError(
                    f"Function names may only be alphanumerics, got {func_name!r}."
                )

        utils.validate_registered_pyfunc(func, duck_typing_okay=True)
        name = func_name or func.__name__
        if name in self._funcs:
            raise ValueError(f"Func[{name}] is already registered with agent.")
        self._funcs[name] = func
        return func

    def _handshake(self) -> AgentMetadata:
        return AgentMetadata(self.base_prompt, self.few_shots)

    def _serve_func(self, func_name: str, query: AgentQuery) -> AgentResponse:
        try:
            pyfunc = self._funcs[func_name]
        except KeyError:
            raise fastapi.HTTPException(
                status_code=404, detail=f"Func[{func_name}] doesn't exist"
            )
        output = pyfunc(query)
        try:
            return _wrap_with_agent_response(output)
        except TypeError:
            raise TypeError(
                f"Func[{func_name}] returned unexpected output of type {type(output)}."
            )


def _wrap_with_agent_response(
    value: Union[str, Message, AgentResponse]
) -> AgentResponse:
    if isinstance(value, str):
        return AgentResponse(Message(value))
    elif isinstance(value, Message):
        return AgentResponse(value)
    elif isinstance(value, AgentResponse):
        return value
    else:
        raise TypeError(f"Unexpected type to wrap: {type(value)}")


def _split_few_shots(few_shots: str) -> List[str]:
    """Split a long string of all few-shots into a list of few-shot strings."""
    # First, strip all lines to remove bad spaces.
    few_shots = "\n".join(line.strip() for line in few_shots.splitlines())
    # Then, split by "\n\nQ:".
    few_shot_splits = few_shots.split("\n\nQ:")
    few_shot_splits = [few_shot_splits[0]] + [
        "Q:" + few_shot for few_shot in few_shot_splits[1:]
    ]
    return few_shot_splits


def _ping_fixie_async(agent_id: str):
    """Asynchronously pings Fixie to refresh the given agent_id."""
    thread = threading.Thread(target=_ping_fixie_sync, args=(agent_id,))
    thread.start()


def _ping_fixie_sync(agent_id: str):
    response = requests.post(f"{constants.FIXIE_REFRESH_URL}/{agent_id}")
    response.raise_for_status()



"""This module holds objects that represent the API interface by which Agents talk to
Fixie ecosystem."""

from __future__ import annotations

import dataclasses
from typing import Optional

from pydantic import dataclasses as pydantic_dataclasses

from fixieai.agents import user_storage


@pydantic_dataclasses.dataclass
class Embed:
    """An Embed represents a binary object attached to a Message."""

    # The MIME content type of the object, e.g., "image/png" or "application/json".
    content_type: str

    # A public URL where the object can be downloaded. The Embed API can be used to
    # upload an Embed object to Fixie and generate a URL.
    uri: str


@pydantic_dataclasses.dataclass
class Message:
    """A Message represents a single message sent to a Fixie agent."""

    # The text of the message.
    text: str

    # A mapping of embed keys to Embed objects.
    embeds: dict[str, Embed] = dataclasses.field(default_factory=dict)


@pydantic_dataclasses.dataclass
class AgentQuery:
    """A standalone query sent to a Fixie agent."""

    # The contents of the query.
    message: Message

    # This is an access token associated with the user for whom this query was
    # created. Agents wishing to make queries to other agents, or to other
    # Fixie services, should carry this token in the query so that it
    # can be tied back to the original user.
    access_token: Optional[str] = None

    def user_storage(self, agent_id: str) -> user_storage.UserStorage:
        """Returns a persistent, user-specific, dict-like memory storage.

        Args:
            agent_id: The agent who's asking for it.

        TODO(hessam): Remove agent_id from the signature once agent_id is included in
         access_token.
        """
        return user_storage.UserStorage(self, agent_id)


@pydantic_dataclasses.dataclass
class AgentResponse:
    """A response message from an Agent."""

    # The text of the response message.
    message: Message



from fixieai.client.client import FixieClient
from fixieai.client.client import get_agents
from fixieai.client.client import get_client
from fixieai.client.client import get_embeds
from fixieai.client.client import query
from fixieai.client.console import Console

__all__ = [
    "FixieClient",
    "get_agents",
    "get_client",
    "get_embeds",
    "query",
    "Console",
]



import enum
import inspect
import re
from typing import TYPE_CHECKING, Callable
from unittest import mock

if TYPE_CHECKING:
    from fixieai.agents import api
    from fixieai.agents import code_shot
else:
    api = mock.MagicMock()
    code_shot = mock.MagicMock()


def strip_prompt_lines(agent_metadata: code_shot.AgentMetadata):
    """Strips all prompt lines."""
    agent_metadata.base_prompt = _strip_all_lines(agent_metadata.base_prompt)
    for i, fewshot in enumerate(agent_metadata.few_shots):
        agent_metadata.few_shots[i] = _strip_all_lines(fewshot)


def validate_code_shot_agent(agent_metadata: code_shot.AgentMetadata):
    """A client-side validation of few_shots and agent."""
    _validate_base_prompt(agent_metadata.base_prompt)
    for fewshot in agent_metadata.few_shots:
        _validate_few_shot_prompt(fewshot)


def validate_registered_pyfunc(func: Callable, duck_typing_okay: bool = False):
    """Validates `func`'s signature to be a valid CodeShot Func.

    Args:
        func: The function to be validated.
        duck_typing_okay: If set, registered function may be duck typed.
    """
    # Validate that func has a proper api.AgentQuery -> api.AgentResponse signature.
    if not inspect.isfunction(func):
        raise TypeError(
            f"Registered function {func!r} is not a function, but a {type(func)!r}."
        )
    signature = inspect.signature(func)
    params = signature.parameters
    if len(params) != 1:
        raise TypeError(
            f"Expected function to get a single argument but it gets {len(params)}."
        )
    param = list(params.values())[0]
    if param.kind not in (param.POSITIONAL_ONLY, param.POSITIONAL_OR_KEYWORD):
        raise TypeError(
            f"Expected function get a positional argument,"
            f" but it gets {param.kind.name!r}."
        )
    param_type = param.annotation
    return_type = signature.return_annotation
    if not duck_typing_okay and inspect.Signature.empty in (param_type, return_type):
        raise TypeError(
            f"Expected registered function to set type annotation for its argument and "
            "return type."
        )
    # Delayed import to avoid circular dependency
    from fixieai.agents import api

    # TODO(hessam): Allow return_type to be a Union of the allowed types.
    if param_type not in (
        api.AgentQuery,
        inspect.Signature.empty,
    ) or return_type not in (
        api.AgentResponse,
        api.Message,
        str,
        inspect.Signature.empty,
    ):
        raise TypeError(
            f"Expected registered function to get a single api.AgentQuery "
            "argument and output an api.AgentResponse but it gets "
            f"{list(params.values())} and returns {return_type}."
        )
    return func


def _strip_all_lines(prompt: str) -> str:
    prompt = prompt.strip()
    return "\n".join(line.strip() for line in prompt.splitlines())


def _validate_base_prompt(base_prompt: str):
    if base_prompt.endswith("\n") or base_prompt.startswith("\n"):
        raise ValueError(
            "base_prompt should not start or end in newlines. "
            f"base_prompt={base_prompt!r}."
        )
    whitespaces = (" ", "\t", "\r")
    prompt_lines = base_prompt.split("\n")
    bad_lines = [
        line
        for line in prompt_lines
        if line.startswith(whitespaces) or line.endswith(whitespaces)
    ]
    if bad_lines:
        raise ValueError(
            f"Some lines in the base prompt start or end in whitespaces: {bad_lines!r}."
        )


class FewshotLinePattern(enum.Enum):
    QUERY = re.compile(r"^Q:")
    AGENT_SAYS = re.compile(r"^Agent\[\w+] says:")
    FUNC_SAYS = re.compile(r"^Func\[\w+] says:")
    ASK_AGENT = re.compile(r"^Ask Agent\[\w+]:")
    ASK_FUNC = re.compile(r"^Ask Func\[\w+]:")
    RESPONSE = re.compile(r"^A:")
    NO_PATTERN: None = None

    @classmethod
    def pattern(cls, line: str) -> "FewshotLinePattern":
        """Returns the matched PromptPattern for a given line."""
        if "\n" in line:
            raise ValueError(
                "Cannot get the pattern for a multi-line text. Patterns must be "
                "extracted one line at a time."
            )
        pattern_matches = [
            prompt_pattern
            for prompt_pattern in cls
            if prompt_pattern is not cls.NO_PATTERN and prompt_pattern.value.match(line)
        ]
        if len(pattern_matches) > 1:
            raise RuntimeError(
                f"More than one pattern ({pattern_matches}) matched the line {line!r}."
            )
        elif len(pattern_matches) == 1:
            return pattern_matches[0]
        else:
            return cls.NO_PATTERN


def _validate_few_shot_prompt(prompt: str):
    """Validates 'prompt' as a correctly formatted few shot prompt."""
    lines = prompt.splitlines(False)

    # Check that no line starts or ends in a whitespace.
    whitespaces = (" ", "\t", "\r")
    bad_lines = [
        line
        for line in lines
        if line.startswith(whitespaces) or line.endswith(whitespaces)
    ]
    _assert(
        not bad_lines,
        f"Some lines in the fewshot start or end in whitespaces: {bad_lines!r}.",
        prompt,
    )

    # Check that it doesn't end with newline
    _assert(not prompt.endswith("\n"), "Fewshot ends with newline.", prompt)

    # Check that fewshot starts with a Q:, ends in an A:, and a Func says and Agent
    # says follows every Ask Func and Ask Agent.
    lines_patterns = [FewshotLinePattern.pattern(line) for line in lines]
    _assert(
        lines_patterns[0] is FewshotLinePattern.QUERY,
        "Fewshot must start with a 'Q:'",
        prompt,
    )
    last_pattern = FewshotLinePattern.QUERY
    for i, pattern in enumerate(lines_patterns):
        if pattern is FewshotLinePattern.ASK_AGENT:
            _assert(
                i + 1 < len(lines_patterns)
                and lines_patterns[i + 1] is FewshotLinePattern.AGENT_SAYS,
                "Each 'Ask Agent' line must be followed by an 'Agent says' line.",
                prompt,
            )
        if pattern is FewshotLinePattern.ASK_FUNC:
            _assert(
                i + 1 < len(lines_patterns)
                and lines_patterns[i + 1] is FewshotLinePattern.FUNC_SAYS,
                "Each 'Ask Func' line must be followed by an 'Func says' line.",
                prompt,
            )
        if pattern is not FewshotLinePattern.NO_PATTERN:
            last_pattern = pattern
    _assert(
        last_pattern is FewshotLinePattern.RESPONSE,
        f"Fewshot must end with a 'A:' pattern, but it ends with {last_pattern}",
        prompt,
    )

    # Check that there's Q: and A: lines are interleaved.
    all_qa_lines = [
        "Q" if pattern is FewshotLinePattern.QUERY else "A"
        for pattern in lines_patterns
        if pattern in (FewshotLinePattern.QUERY, FewshotLinePattern.RESPONSE)
    ]
    qa_str = "".join(all_qa_lines)
    _assert(
        qa_str.replace("QA", "") == "",
        "Q: and A: lines must be interleaved in fewshot.",
        prompt,
    )


def _assert(condition: bool, msg: str, prompt: str):
    if not condition:
        raise ValueError(f"{msg} in few-shot prompt: {prompt!r}.")



#!/usr/bin/env python3

import click
import prompt_toolkit
import requests
import rich.console as rich_console

from fixieai.client import FixieClient

HISTORY_FILE = "history.txt"

textconsole = rich_console.Console()


class Console:
    """A simple console interface for Fixie."""

    def __init__(
        self,
        client: FixieClient,
        history_file: str = HISTORY_FILE,
    ):
        self._client = client
        self._session = client.create_session()
        self._history_file = history_file
        self._response_index = 0

    def run(self) -> None:
        """Run the console application."""

        textconsole.print("[blue]Welcome to Fixie!")
        textconsole.print(f"Connected to: {self._session.session_url}")
        while True:
            in_text = prompt_toolkit.prompt(
                "fixie 🚲❯ ",
                history=prompt_toolkit.history.FileHistory(self._history_file),
                auto_suggest=prompt_toolkit.auto_suggest.AutoSuggestFromHistory(),
            )
            self._query(in_text)

    def _query(self, in_text: str) -> None:
        with textconsole.status("Working...", spinner="bouncingBall"):
            try:
                self._response_index += 1
                for message in self._session.run(in_text):
                    if message["type"] != "response":
                        textconsole.print(
                            f"   [dim]@{message['sentBy']}: {message['text']}[/]"
                        )
                    else:
                        textconsole.print(f"{self._response_index}❯ {message['text']}")
            except requests.exceptions.HTTPError as e:
                textconsole.print(f"🚨 {e}")
                return


@click.group()
@click.option(
    "--fixie_api_url",
    help="Fixie API URL. Defaults to the value of the FIXIE_API_URL env var, "
    "or `https://app.fixie.ai` if that is unset.",
)
@click.option("--verbose", is_flag=True, help="Enable verbose output.")
@click.pass_context
def fixie(ctx, fixie_api_url, verbose):
    ctx.ensure_object(dict)
    client = FixieClient(api_url=fixie_api_url)
    ctx.obj["fixie_API_URL"] = fixie_api_url
    ctx.obj["CLIENT"] = client
    ctx.obj["VERBOSE"] = verbose


@fixie.command("console", help="Run live console.")
@click.pass_context
def console(ctx):
    client = ctx.obj["CLIENT"]
    c = Console(client)
    c.run()


@fixie.group(help="Agent-related commands.")
def agents():
    pass


@agents.command("list", help="List agents.")
@click.pass_context
def list_agents(ctx):
    client = ctx.obj["CLIENT"]
    agents = client.get_agents()
    for agent_id, agent in agents.items():
        textconsole.print(f"[green]{agent_id}[/]: {agent['name']}")
        if ctx.obj["VERBOSE"]:
            textconsole.print(f"    {agent['description']}")
            textconsole.print(f"    [yellow]Developer[/]: {agent['developer']}")
            if "moreInfo" in agent and agent["moreInfo"]:
                textconsole.print(f"    [yellow]More info[/]: {agent['moreInfo']}")
            textconsole.print("")


@fixie.group(help="Session-related commands.")
def sessions():
    pass


@sessions.command("list", help="List sessions.")
@click.pass_context
def list_sessions(ctx):
    client = ctx.obj["CLIENT"]
    session_ids = client.get_sessions()
    for session_id in session_ids:
        textconsole.print(f"[green]{session_id}[/]")


@sessions.command("show", help="Show session.")
@click.pass_context
@click.argument("session_id")
def show_session(ctx, session_id: str):
    client = ctx.obj["CLIENT"]
    session = client.get_session(session_id)
    messages = session.get_messages()
    textconsole.print(messages)


@sessions.command("embeds", help="Show embeds in a session.")
@click.pass_context
@click.argument("session_id")
def embeds(ctx, session_id: str):
    client = ctx.obj["CLIENT"]
    session = client.get_session(session_id)
    embeds = session.get_embeds()
    textconsole.print(embeds)


if __name__ == "__main__":
    fixie()



import pytest

from fixieai.agents import utils

BAD_PROMPTS = [
    # Ask Agent without Agent says
    """Q: Who is the president?
Ask Agent[search]: Who is the president?
A: It's Joe Biden.""",
    # Doesn't start with a query
    """Ask Agent[search]: Who is the president?
Q: Who is the president?
A: It's Joe Biden.""",
    # Ask Func without Func says
    """Q: How many books are there in the library?
Ask Func[library]: How many books are there in the library?
A: I don't know.""",
    # Doesn't end with a response
    """Q: What is 12 + 15?
Agent[calc] says: 27""",
    # 2 queries
    """Q: What is the life expectancy in the US?
Q: Do we need to call an agent?
A: No we don't.""",
    # 2 responses
    """Q: What is the life expectancy in the US?
A: I think it's 80 years but I'm not sure.
A: I'm not sure.""",
    # extra whitespace
    """Q: What is the life expectancy in the US?
A: I think it's 80 years but I'm not sure. """,
    # empty line at the end
    """Q: What is the life expectancy in the US?
A: I think it's 80 years but I'm not sure.
""",
    # empty line to start
    """
Q: What is the life expectancy in the US?
    A: I think it's 80 years but I'm not sure.""",
]


@pytest.mark.parametrize("bad_prompt", BAD_PROMPTS)
def test_fewshot_prompt_fails_with_bad_prompt(bad_prompt):
    with pytest.raises(ValueError):
        utils._validate_few_shot_prompt(bad_prompt)


GOOD_PROMPTS = [
    """Q: Who is the president?
Ask Agent[search]: Who is the president?
Agent[search] says: It's Joe Biden.
A: It's Joe Biden.""",
    # Ask Func without Func says
    """Q: How many books are there in the library?
Ask Func[library]: How many books are there in the library?
Func[library] says: None
A: I don't know.""",
    """Q: What is 12 + 15?
Agent[calc] says: 27
A: It's 27.""",
    """Q: What is the life expectancy in the US?
A: I think it's 80 years but I'm not sure.""",
    """Q: Replace the background in [image1] with a beautiful sunset
Ask Agent[mask2former]: Mask out the background in [image1]
Agent[mask2former] says: I have masked out the selected region: [mask1]
Ask Func[edit]: A beautiful sunset [image1] [mask1]
Func[edit] says: Here you go: [image2]
A: I replaced the background with a beautiful sunset: [image2]""",
]


@pytest.mark.parametrize("good_prompt", GOOD_PROMPTS)
def test_fewshot_prompt_succeeds_with_good_prompt(good_prompt):
    utils._validate_few_shot_prompt(good_prompt)



"""This file holds some configuration constants, including URL to Fixie services,
and your Fixie API key."""

import os

# Base Fixie platform URL.
FIXIE_API_URL = os.getenv("FIXIE_API_URL", "https://app.fixie.ai")
# Your Fixie API key, set from environment variables.
FIXIE_API_KEY = os.getenv("FIXIE_API_KEY")

# Fixie's UserStorage service URL.
FIXIE_USER_STORAGE_URL = f"{FIXIE_API_URL}/api/userstorage"
# Fixie's refresh endpoint. It will be pinged when your agent comes alive.
FIXIE_REFRESH_URL = f"{FIXIE_API_URL}/api/refresh"



import pytest
import requests_mock

from fixieai.client.client import FixieClient


@pytest.fixture
def fixie_client():
    with requests_mock.Mocker() as m:
        m.post(
            "https://test.fixie.ai/graphql",
            json={"data": {"createSession": {"session": {"handle": "test-handle"}}}},
        )
        return FixieClient(api_url="https://test.fixie.ai", api_key="test-key")


def test_agents(fixie_client):
    with requests_mock.Mocker() as m:
        m.post(
            "https://test.fixie.ai/graphql",
            json={"data": {"allAgents": [{"handle": "test", "name": "Test Agent"}]}},
        )
        assert fixie_client.get_agents() == {
            "test": {"handle": "test", "name": "Test Agent"}
        }


def test_sessions(fixie_client):
    with requests_mock.Mocker() as m:
        m.post(
            "https://test.fixie.ai/graphql",
            json={
                "data": {
                    "allSessions": [{"handle": "test-handle", "name": "Test Session"}]
                }
            },
        )
        assert fixie_client.get_sessions() == ["test-handle"]


def test_get_session():
    with requests_mock.Mocker() as m:
        m.post(
            "https://test.fixie.ai/graphql",
            json={
                "data": {
                    "sessionByHandle": {
                        "handle": "test-handle",
                    }
                }
            },
        )
        client = FixieClient(
            api_url="https://test.fixie.ai",
            api_key="test-key",
        )
        session = client.get_session("test-handle")

        m.post(
            "https://test.fixie.ai/graphql",
            json={
                "data": {
                    "sessionByHandle": {
                        "handle": "test-handle",
                        "messages": [
                            {
                                "id": 1,
                                "text": "Test message",
                            }
                        ],
                    }
                }
            },
        )
        assert session.get_messages() == [{"id": 1, "text": "Test message"}]
